#ifndef equations_h
#define equations_h

////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef ARRL
#define EQUATION "ARRL"

#define VARS \
  X(A) \
  X(B) \
  X(C)
#define LAST s_C

#define VARSI \
  X(C) \
  X(B) \
  X(A)

#define DEN (r->B*l->R + r->C*l->LEN) * 1.0E6
#define NUM r->A * sqrd(l->R) * sqrd(l->N)


////////////////////////////////////////////////////////////////////////////////////////////////////
#elif defined(SIMPLE)
#define EQUATION "SIMPLE"

// The variables.
#define VARS \
  X(r0) \
  X(r1) \
  X(r2) \
  X(r3) \
  X(cd) \
  X(t0) \
  X(t1) \
  X(t2) \
  X(t3)

#define LAST s_t3

// In reverse order.
#define VARSI \
  X(t3) \
  X(t2) \
  X(t1) \
  X(t0) \
  X(cd) \
  X(r3) \
  X(r2) \
  X(r1) \
  X(r0)

// The equation denumerator.
#define DEN  r->cd

// The equation nominator.
//#define NUM  u0 * (r->r0 + r->r1*l->R + r->r2*sqrd(l->R) + r->r3*cubed(l->R)) * \

//    (r->t0 + r->t1*l->N + r->t2*sqrd(l->N) + r->t3*cubed(l->N));
#define NUM u0 * (l->R * (l->R * (l->R*r->r3 + r->r2) + r->r1) + r->r0) * \
  (l->N * (l->N * (l->N*r->t3 + r->t2) + r->t1) + r->t0)

////////////////////////////////////////////////////////////////////////////////////////////////////
#elif defined(EQUATION8)
#define EQUATION "EQUATION 8"
#define VARS \
  X(rd0) \
  X(rd1) \
  X(rd2) \
  X(l1) \
  X(l2) \
  X(l3) \
  X(rn0) \
  X(rn1) \
  X(rn2) \
  X(rn3) \
  X(t0) \
  X(t1) \
  X(t2)

#define LAST s_t2

#define VARSI \
  X(t2) \
  X(t1) \
  X(t0) \
  X(rn3) \
  X(rn2) \
  X(rn1) \
  X(rn0) \
  X(l3) \
  X(l2) \
  X(l1) \
  X(rd2) \
  X(rd1) \
  X(rd0)
  
#define DEN (r->rd0 + r->rd1*l->R + r->rd2*(sqrd(l->R)) + \
	     r->l1*l->LEN + r->l2*sqrd(l->LEN) + r->l3*cubed(l->LEN))
#define NUM u0 * (r->rn0 + r->rn1*l->R + r->rn2*sqrd(l->R) + r->rn3*cubed(l->R)) \
  * (r->t0 + r->t1*l->N + r->t2*sqrd(l->N))
  
#elif defined(EQUATIONA)
#define EQUATION  "EQUATION A"
#define VARS \
  X(r0) \
  X(r1) \
  X(r2) \
  X(r3) \
  X(r4) \
  X(r5) \
  X(t0) \
  X(t1) \
  X(t2) \
  X(l0) \
  X(l1) \
  X(l2) \
  X(l3) \
  X(l4) \
  X(l5)

#define LAST s_l5

#define VARSI \
  X(l5) \
  X(l4) \
  X(l3) \
  X(l2) \
  X(l1) \
  X(l0) \
  X(t2) \
  X(t1) \
  X(t0) \
  X(r5) \
  X(r4) \
  X(r3) \
  X(r2) \
  X(r1) \
  X(r0)

//#define DEN (r->l0 + r->l1*l->LEN + r->l2*sqrd(l->LEN) + r->l3*cubed(l->LEN) + \
//	     r->l4*quartd(l->LEN) + r->l5*quintd(l->LEN))
#define DEN (l->LEN * (l->LEN * (l->LEN * (l->LEN * (l->LEN*r->l5 + r->l4) + r->l3) + r->l2) + \
		       r->l1) + r->l0)
//#define NUM u0 * (r->r0 + r->r1*l->R + r->r2*sqrd(l->R) + r->r3*cubed(l->R) + \
//		  r->r4*quartd(l->R) + r->r5*quintd(l->R)) * \
//  (r->t0 + r->t1*l->N + r->t2*sqrd(l->N))
#define NUM u0 * ((l->R * (l->R * (l->R * (l->R * (l->R*r->r5 + r->r4) + r->r3) + r->r2) + \
			   r->r1) + r->r0) *				\
		  (l->N * (l->N*r->t2 + r->t1) + r->t0))

#elif defined(EQUATIONC)
#define EQUATION "EQUATION C"
#define VARS \
  X(r0) \
  X(r1) \
  X(r2) \
  X(r3) \
  X(r4) \
  X(r5) \
  X(t0) \
  X(t1) \
  X(t2) \
  X(w0) \
  X(w1) \
  X(w2) \
  X(l0) \
  X(l1) \
  X(l2) \
  X(l3) \
  X(l4) \
  X(l5)

#define LAST s_l5

#define VARSI \
  X(l5) \
  X(l4) \
  X(l3) \
  X(l2) \
  X(l1) \
  X(l0) \
  X(w2) \
  X(w1) \
  X(w0) \
  X(t2) \
  X(t1) \
  X(t0) \
  X(r5) \
  X(r4) \
  X(r3) \
  X(r2) \
  X(r1) \
  X(r0)

//#define DEN (r->l0 + r->l1*l->LEN + r->l2*sqrd(l->LEN) + r->l3*cubed(l->LEN) + \
//	     r->l4*quartd(l->LEN) + r->l5*quintd(l->LEN))
#define DEN (l->LEN * (l->LEN * (l->LEN * (l->LEN * (l->LEN*r->l5 + r->l4) + r->l3) + r->l2) + \
		       r->l1) + r->l0)

//#define NUM u0 * (r->r0 + r->r1*l->R + r->r2*sqrd(l->R) + r->r3*cubed(l->R) + \
//		    r->r4*quartd(l->R) + r->r5*quintd(l->R)) *	\
//  (r->t0 + r->t1*l->N + r->t2*sqrd(l->N)) * \
//  (r->w0 + r->w1*l->W + r->w2*sqrd(l->W))
#define NUM u0 * ((l->R * (l->R * (l->R * (l->R * (l->R*r->r5 + r->r4) + r->r3) + r->r2) + \
			   r->r1) + r->r0) *				\
		  (l->N * (l->N*r->t2 + r->t1) + r->t0) * \
  		  (l->W * (l->W*r->w2 + r->w1) + r->w0))
  

////////////////////////////////////////////////////////////////////////////////////////////////////
#elif defined(EQUATIOND)
 #define EQUATION "EQUATION D"
#define VARS \
  X(r0) \
  X(r1) \
  X(r2) \
  X(r3) \
  X(t0) \
  X(t1) \
  X(t2) \
  X(w0) \
  X(w1) \
  X(w2) \
  X(l0) \
  X(l1) \
  X(l2) \
  X(l3)

#define LAST s_l3

#define VARSI \
  X(l3) \
  X(l2) \
  X(l1) \
  X(l0) \
  X(w2) \
  X(w1) \
  X(w0) \
  X(t2) \
  X(t1) \
  X(t0) \
  X(r3) \
  X(r2) \
  X(r1) \
  X(r0)

#define DEN r->l0 + r->l1*l->LEN + r->l2*sqrd(l->LEN) + r->l3*cubed(l->LEN)
//#define DEN (l->LEN * (l->LEN * (l->LEN*r->l3 + r->l2) + r->l1) + r->l0)

#define NUM u0 * (r->r0 + r->r1*l->R + r->r2*sqrd(l->R) + r->r3*cubed(l->R)) * \
    (r->t0 + r->t1*l->N + r->t2*sqrd(l->N)) * (r->w0 + r->w1*l->W + r->w2*sqrd(l->W));
  
//#define NUM u0 * (l->R * (l->R * (l->R*r->r3 + r->r2) + r->r1) + r->r0) * \
//  (l->N * (l->N*r->t2 + r->t1) + r->t0) * \
//  (l->W * (l->W*r->w2 + r->w1) + r->w0)

////////////////////////////////////////////////////////////////////////////////////////////////////
#elif defined(EQUATIONE)
#define EQUATION "EQUATION E"
#define VARS \
  X(l0) \
  X(l1) \
  X(l2) \
  X(l3) \
  X(l4) \
  X(l5) \
  X(r0) \
  X(r1) \
  X(t0) \
  X(t1) \
  X(t2)

#define LAST s_t2
  
#define VARSI \
  X(t2) \
  X(t1) \
  X(t0) \
  X(r1) \
  X(r0) \
  X(l5) \
  X(l4) \
  X(l3) \
  X(l2) \
  X(l1) \
  X(l0)
  
#define DEN (r->l0 + r->l1*l->LEN + r->l2*sqrd(l->LEN) + r->l3*cubed(l->LEN) + \
            r->l4*quartd(l->LEN) + r->l5*quintd(l->LEN))
#define NUM u0 * (r->r0 * log(r->r1 * l->R)) * (r->t0 + r->t1*l->N + r->t2*sqrd(l->N))

#elif defined(EQUATIONF)
#define EQUATION "EQUATION F"
#define VARS \
  X(r0) \
  X(r1) \
  X(r2) \
  X(r3) \
  X(t0) \
  X(t1) \
  X(t2) \
  X(l0) \
  X(l1) \
  X(l2) \
  X(l3)

#define LAST s_l3

#define VARSI \
  X(l3) \
  X(l2) \
  X(l1) \
  X(l0) \
  X(t2) \
  X(t1) \
  X(t0) \
  X(r3) \
  X(r2) \
  X(r1) \
  X(r0)

#define DEN r->l0 + r->l1*l->LEN + r->l2*sqrd(l->LEN) + r->l3*cubed(l->LEN)
#define NUM u0 * (r->r0 + r->r1*l->R + r->r2*sqrd(l->R) + r->r3*cubed(l->R)) * \
  (r->t0 + r->t1*l->N + r->t2*sqrd(l->N))
	
#elif defined(EQUATIONG)
#define EQUATION "EQUATION G"
#define VARS \
  X(rn0) \
  X(rn1) \
  X(rn2) \
  X(rn3) \
  X(cd) \
  X(rd1) \
  X(rd2) \
  X(t0) \
  X(t1) \
  X(t2) \
  X(l1) \
  X(l2) \
  X(l3)

#define LAST s_l3
	
#define VARSI  \
  X(l3) \
  X(l2) \
  X(l1) \
  X(t2) \
  X(t1) \
  X(t0) \
  X(rd2) \
  X(rd1) \
  X(cd) \
  X(rn3) \
  X(rn2) \
  X(rn1) \
  X(rn0)

#define DEN r->cd + r->rd1*l->R + r->rd2*sqrd(l->R) + \
    r->l1*l->LEN + r->l2*sqrd(l->LEN) + r->l3*cubed(l->LEN)
#define NUM u0 * (r->rn0 + r->rn1*l->R + r->rn2*sqrd(l->R) + r->rn3*cubed(l->R)) * \
	(r->t0 + r->t1*l->N + r->t2*sqrd(l->N));

#elif defined(EQUATIONH)
#define EQUATION "EQUATION H"

#define VARS \
  X(r0) \
  X(r1) \
  X(r2) \
  X(r3) \
  X(l0) \
  X(l1) \
  X(l2) \
  X(l3)

#define LAST s_l3

#define VARSI \
  X(l3) \
  X(l2) \
  X(l1) \
  X(l0) \
  X(r3) \
  X(r2) \
  X(r1) \
  X(r0)

#define DEN (r->r0 + r->r1*l->R + r->r2*sqrd(l->R) + r->r3*cubed(l->R))
#define NUM u0 * (r->l0 + r->l1*l->LEN + r->l2*sqrd(l->LEN) + r->l3*cubed(l->LEN))

#elif defined(EQUATIONI)
#define EQUATION "EQUATION I"
#define VARS \
  X(r0) \
    X(r1) \
    X(r2) \
    X(r3) \
    X(w0) \
    X(w1) \
    X(w2) \
    X(w3) \
    X(t0) \
    X(t1) \
    X(t2) \
    X(t3) \
    X(l0) \
    X(l1) \
    X(l2) \
    X(l3)

#define LAST s_l3

#define VARSI \
  X(l3) \
    X(l2) \
    X(l1) \
    X(l0) \
    X(t3) \
    X(t2) \
    X(t1) \
    X(t0) \
    X(w3) \
    X(w2) \
    X(w1) \
    X(w0) \
    X(r3) \
    X(r2) \
    X(r1) \
    X(r0) \

//#define NUM u0 * (r->r0 + r->r1*l->R + r->r2*sqrd(l->R) + r3*cubed(l->R)) * \
//  (r->w0 + r->w1*l->W + r->w2*sqrd(l->W) + r->w3*cubed(l->W)) * \
//  (r->t0 + r->t1*l->N + r->t2*sqri(l->N) + r->t3*cubed(l->N))
// Note: moved wire gauge to denominator for fun.
// A faster version we should have been using all along.
#define NUM u0 * (l->R * (l->R * (l->R*r->r3 + r->r2) + r->r1) + r->r0) * \
  (l->N * (l->N * (l->N*r->t3 + r->t2) + r->t1) + r->t0)
  
//#define DEN (r->l0 + r->l1*l->LEN + r->l2*sqrd(l->LEN) + r->l3*cubed(l->LEN))
#define DEN (l->LEN * (l->LEN * (l->LEN*r->l3 + r->l2) + r->l1) + r->l0) * \
    (l->W * (l->W * (l->W*r->w3 + r->w2) + r->w1) + r->w0)

#elif defined(EQUATIONJ)
#define EQUATION "EQUATION J"
#define VARS \
  X(r0) \
    X(r1) \
    X(r2) \
    X(r3) \
    X(w0) \
    X(w1) \
    X(w2) \
    X(t0) \
    X(t1) \
    X(t2) \
    X(t3) \
    X(l0) \
    X(l1) \
    X(l2) \
    X(l3)

#define LAST s_l3

#define VARSI \
  X(l3) \
    X(l2) \
    X(l1) \
    X(l0) \
    X(t3) \
    X(t2) \
    X(t1) \
    X(t0) \
    X(w2) \
    X(w1) \
    X(w0) \
    X(r3) \
    X(r2) \
    X(r1) \
    X(r0) \

//#define NUM u0 * (r->r0 + r->r1*l->R + r->r2*sqrd(l->R) + r3*cubed(l->R)) * \
//  (r->w0 + r->w1*l->W + r->w2*sqrd(l->W)) * \
//  (r->t0 + r->t1*l->N + r->t2*sqri(l->N) + r->t3*cubed(l->N))
// Note: moved wire gauge to denominator for fun.
// A faster version we should have been using all along.
#define NUM u0 * (l->R * (l->R * (l->R*r->r3 + r->r2) + r->r1) + r->r0) * \
  (l->N * (l->N * (l->N*r->t3 + r->t2) + r->t1) + r->t0)
  
//#define DEN (r->l0 + r->l1*l->LEN + r->l2*sqrd(l->LEN) + r->l3*cubed(l->LEN))
#define DEN (l->LEN * (l->LEN * (l->LEN*r->l3 + r->l2) + r->l1) + r->l0) * \
  (l->W * (l->W * r->w2 + r->w1) + r->w0)

#else
  #error No known equation specified
#endif



// A result. These are the coefficients of the polynomial for inductance and are being tested by
// the genetic algorithm optimizer.
#define X(nme) double nme;
typedef struct {
  void   *next;                 // For storing in lists.
  double goodness;              // The goodness measurement. 0.0 is best.
  VARS
  uint32_t flags;               // A bit on means search around this parameters major value.
} result;

#undef X

#define X(nme) s_##nme,
enum {VARS};
#undef X

#endif




